@@ -21,13 +21,13 @@ class CrashDatabase(apport.crashdb.CrashDatabase):
     
     This is mainly useful for testing and debugging.'''
 
-    def __init__(self, auth_file, bugpattern_baseurl, options):
+    def __init__(self, auth_file, bugpattern_url, options):
         '''Initialize crash database connection.
         
         This class does not support bug patterns and authentication.'''
 
         apport.crashdb.CrashDatabase.__init__(self, auth_file,
-            bugpattern_baseurl, options)
+            bugpattern_url, options)
 
         self.reports = [] # list of dictionaries with keys: report, fixed_version, dup_of, comment
         self.unretraced = set()
@@ -329,6 +329,38 @@ class _T(unittest.TestCase):
         self.assertEqual(self.crashes.get_unretraced(), set([0, 1, 2]))
         self.assertEqual(self.crashes.get_dup_unchecked(), set([3, 4]))
 
+    def test_dynamic_crashdb_conf(self):
+        '''Dynamic code in crashdb.conf'''
+
+        # use our dummy crashdb
+        crashdb_conf = tempfile.NamedTemporaryFile()
+        print >> crashdb_conf, '''default = 'testsuite'
+
+def get_dyn():
+    return str(2 + 2)
+
+def get_dyn_name():
+    return 'on_the' + 'fly'
+
+databases = {
+    'testsuite': { 
+        'impl': 'memory',
+        'dyn_option': get_dyn(),
+    },
+    get_dyn_name(): {
+        'impl': 'memory',
+        'whoami': 'dynname',
+    }
+}
+'''
+        crashdb_conf.flush()
+
+        db = apport.crashdb.get_crashdb(None, None, crashdb_conf.name)
+        self.assertEqual(db.options['dyn_option'], '4')
+        db = apport.crashdb.get_crashdb(None, 'on_thefly', crashdb_conf.name)
+        self.assertFalse('dyn_opion' in db.options)
+        self.assertEqual(db.options['whoami'], 'dynname')
+
     #
     # Test memory.py implementation
     #
@@ -394,7 +426,7 @@ class _T(unittest.TestCase):
         self.assertEqual(self.crashes.reports[1]['comment'], 'muhaha')
         self.assertEqual(self.crashes.download(1)['Package'], 'libfoo1 1.2-4')
         self.assertEqual(self.crashes.download(1)['StacktraceTop'], 'Fresh!')
-        self.failIf(self.crashes.download(1).has_key('FooBar'))
+        self.assertFalse(self.crashes.download(1).has_key('FooBar'))
 
         self.assertRaises(IndexError, self.crashes.update_traces, 5, None)
 
@@ -552,6 +584,33 @@ class _T(unittest.TestCase):
         self.assertEqual(self.crashes.check_duplicate(5), None)
         self.assertEqual(self.crashes.check_duplicate(6), (5, None))
 
+    def test_check_duplicate_custom_signature(self):
+        '''check_duplicate() with custom DuplicateSignature: field'''
+
+        r = apport.Report()
+        r['SourcePackage'] = 'bash'
+        r['Package'] = 'bash 5'
+        r['DuplicateSignature'] = 'Code42Blue'
+        self.assertEqual(self.crashes.get_comment_url(r, self.crashes.upload(r)),
+            'http://bash.bugs.example.com/5')
+
+        self.crashes.init_duplicate_db(':memory:')
+        self.assertEqual(self.crashes.check_duplicate(5), None)
+
+        self.assertEqual(self.crashes._duplicate_db_dump(), {'Code42Blue': (5, None)})
+
+        # this one has a standard crash_signature
+        self.assertEqual(self.crashes.check_duplicate(0), None)
+        # ... but DuplicateSignature wins
+        self.crashes.download(0)['DuplicateSignature'] = 'Code42Blue'
+        self.assertEqual(self.crashes.check_duplicate(0), (5, None))
+
+        self.crashes.download(1)['DuplicateSignature'] = 'CodeRed'
+        self.assertEqual(self.crashes.check_duplicate(1), None)
+        self.assertEqual(self.crashes._duplicate_db_dump(), 
+                {'Code42Blue': (5, None), 'CodeRed': (1, None), 
+                 self.crashes.download(0).crash_signature(): (0, None)})
+
     def test_check_duplicate_report_arg(self):
         '''check_duplicate() with explicitly passing report'''
 
@@ -605,7 +664,7 @@ ZeroDivisionError%i: integer division or modulo by zero''' % bug
         if pid == 0:
             try:
                 self.crashes.duplicate_db_consolidate()
-            except Exception, e:
+            except Exception as e:
                 if 'database is locked' in str(e):
                     os._exit(42)
                 else:
@@ -614,7 +673,7 @@ ZeroDivisionError%i: integer division or modulo by zero''' % bug
 
         try:
             self.crashes.duplicate_db_consolidate()
-        except Exception, e:
+        except Exception as e:
             if 'database is locked' in str(e):
                 locked_exceptions += 1
             else:
@@ -622,7 +681,7 @@ ZeroDivisionError%i: integer division or modulo by zero''' % bug
 
         # wait on child, examine status
         status = os.wait()[1]
-        self.assert_(os.WIFEXITED(status))
+        self.assertTrue(os.WIFEXITED(status))
         status = os.WEXITSTATUS(status)
         if status == 42:
             locked_exceptions += 1
@@ -668,19 +727,19 @@ ZeroDivisionError%i: integer division or modulo by zero''' % bug
         self.crashes.init_duplicate_db(':memory:')
 
         # a fresh and empty db does not need consolidation
-        self.failIf(self.crashes.duplicate_db_needs_consolidation())
+        self.assertFalse(self.crashes.duplicate_db_needs_consolidation())
 
         time.sleep(1.1)
         # for an one-day interval we do not need consolidation
-        self.failIf(self.crashes.duplicate_db_needs_consolidation())
+        self.assertFalse(self.crashes.duplicate_db_needs_consolidation())
         # neither for a ten second one (check timezone offset errors)
-        self.failIf(self.crashes.duplicate_db_needs_consolidation(10))
+        self.assertFalse(self.crashes.duplicate_db_needs_consolidation(10))
         # but for an one second interval
-        self.assert_(self.crashes.duplicate_db_needs_consolidation(1))
+        self.assertTrue(self.crashes.duplicate_db_needs_consolidation(1))
 
         self.crashes.duplicate_db_consolidate()
 
-        self.failIf(self.crashes.duplicate_db_needs_consolidation(1))
+        self.assertFalse(self.crashes.duplicate_db_needs_consolidation(1))
 
     def test_change_master_id(self):
         '''duplicate_db_change_master_id()'''
@@ -728,7 +787,7 @@ ZeroDivisionError%i: integer division or modulo by zero''' % bug
             self.assertEqual(self.crashes._duplicate_db_dump(), 
                 {self.crashes.download(0).crash_signature(): (0, '42')})
 
-            self.failIf(self.crashes.duplicate_db_needs_consolidation())
+            self.assertFalse(self.crashes.duplicate_db_needs_consolidation())
             del self.crashes
 
             # damage file
@@ -737,7 +796,7 @@ ZeroDivisionError%i: integer division or modulo by zero''' % bug
             f.close()
 
             self.crashes = CrashDatabase(None, None, {})
-            self.assertRaises(SystemError, self.crashes.init_duplicate_db, db)
+            self.assertRaises(StandardError, self.crashes.init_duplicate_db, db)
 
         finally:
             os.unlink(db)
