@@ -9,10 +9,17 @@
 # option) any later version.  See http://www.gnu.org/copyleft/gpl.html for
 # the full text of the license.
 
-import os, glob, subprocess, os.path, ConfigParser
+import os, glob, subprocess, os.path
+
+try:
+    from configparser import ConfigParser, NoOptionError, NoSectionError
+except ImportError:
+    # Python 2
+    from ConfigParser import ConfigParser, NoOptionError, NoSectionError
+
 from problem_report import ProblemReport
 
-from packaging_impl import impl as packaging
+from apport.packaging_impl import impl as packaging
 
 report_dir = os.environ.get('APPORT_REPORT_DIR', '/var/crash')
 
@@ -196,7 +203,7 @@ def make_report_path(report, uid=None):
     elif report.has_key('Package'):
         subject = report['Package'].split(None, 1)[0]
     else:
-        raise ValueError, 'report has neither ExecutablePath nor Package attribute'
+        raise ValueError('report has neither ExecutablePath nor Package attribute')
 
     if not uid:
         uid = os.getuid()
@@ -234,7 +241,7 @@ def get_config(section, setting, default=None, bool=False):
     interpreted as a boolean.
     '''
     if not get_config.config:
-        get_config.config = ConfigParser.ConfigParser()
+        get_config.config = ConfigParser()
         get_config.config.read(os.path.expanduser(_config_file))
 
     try:
@@ -242,7 +249,7 @@ def get_config(section, setting, default=None, bool=False):
             return get_config.config.getboolean(section, setting)
         else:
             return get_config.config.get(section, setting)
-    except (ConfigParser.NoOptionError, ConfigParser.NoSectionError):
+    except (NoOptionError, NoSectionError):
         return default
 
 get_config.config = None
@@ -252,7 +259,10 @@ get_config.config = None
 #
 
 import unittest, tempfile, os, shutil, sys, time
-from cStringIO import StringIO
+try:
+    from cStringIO import StringIO
+except ImportError:
+    from io import StringIO
 
 class _T(unittest.TestCase):
     def setUp(self):
@@ -278,8 +288,8 @@ class _T(unittest.TestCase):
 
         open(r1, 'w').write('report 1')
         open(r2, 'w').write('report 2')
-        os.chmod(r1, 0600)
-        os.chmod(r2, 0600)
+        os.chmod(r1, 0o600)
+        os.chmod(r2, 0o600)
         if create_inaccessible:
             ri = os.path.join(report_dir, 'inaccessible.crash')
             open(ri, 'w').write('inaccessible')
@@ -321,8 +331,8 @@ class _T(unittest.TestCase):
         if onedesktop:
             d = find_package_desktopfile(onedesktop)
             self.assertNotEqual(d, None, 'one-desktop package %s' % onedesktop)
-            self.assert_(os.path.exists(d))
-            self.assert_(d.endswith('.desktop'))
+            self.assertTrue(os.path.exists(d))
+            self.assertTrue(d.endswith('.desktop'))
 
     def test_likely_packaged(self):
         '''likely_packaged().'''
@@ -443,9 +453,9 @@ CrashCounter: 3''' % time.ctime(time.mktime(time.localtime())-3600))
         self.assertRaises(ValueError, make_report_path, pr)
 
         pr['Package'] = 'bash 1'
-        self.assert_(make_report_path(pr).startswith('%s/bash' % report_dir))
+        self.assertTrue(make_report_path(pr).startswith('%s/bash' % report_dir))
         pr['ExecutablePath'] = '/bin/bash';
-        self.assert_(make_report_path(pr).startswith('%s/_bin_bash' % report_dir))
+        self.assertTrue(make_report_path(pr).startswith('%s/_bin_bash' % report_dir))
 
     def test_check_files_md5(self):
         '''check_files_md5().'''
