@@ -1,6 +1,6 @@
 '''Convenience functions for use in package hooks.'''
 
-# Copyright (C) 2008 - 2010 Canonical Ltd.
+# Copyright (C) 2008 - 2011 Canonical Ltd.
 # Authors: 
 #   Matt Zimmerman <mdz@canonical.com>
 #   Brian Murray <brian@ubuntu.com>
@@ -21,6 +21,8 @@ import re
 import string
 import stat
 import base64
+import tempfile
+import shutil
 
 import xml.dom, xml.dom.minidom
 
@@ -51,7 +53,7 @@ def read_file(path):
     '''
     try:
         return open(path).read().strip()
-    except Exception, e:
+    except Exception as e:
         return 'Error: ' + str(e)
 
 def attach_file(report, path, key=None):
@@ -63,13 +65,28 @@ def attach_file(report, path, key=None):
     if not key:
         key = path_to_key(path)
 
+    # Do not clobber existing keys
+    while report.has_key(key):
+        key += "_"
     report[key] = read_file(path)
 
 def attach_dmesg(report):
-    '''Attach information from the kernel ring buffer (dmesg).'''
+    '''Attach information from the kernel ring buffer (dmesg).
 
-    report['BootDmesg'] = open('/var/log/dmesg').read()
-    report['CurrentDmesg'] = command_output(['sh', '-c', 'dmesg | comm -13 --nocheck-order /var/log/dmesg -'])
+    This won't overwite already existing information.
+    '''
+    try:
+        if not report.get('BootDmesg', '').strip():
+            report['BootDmesg'] = open('/var/log/dmesg').read()
+    except IOError:
+        pass
+    if not report.get('CurrentDmesg', '').strip():
+        dmesg = command_output(['sh', '-c', 'dmesg | comm -13 --nocheck-order /var/log/dmesg -'])
+        # if an initial message was truncated by the ring buffer, skip over it
+        first_newline = dmesg.find('\n[')
+        if first_newline != -1:
+            dmesg = dmesg[first_newline+1:]
+        report['CurrentDmesg'] = dmesg
 
 def attach_dmi(report):
     dmi_dir = '/sys/class/dmi/id'
@@ -109,11 +126,12 @@ def attach_hardware(report):
 
     attach_file(report, '/proc/interrupts', 'ProcInterrupts')
     attach_file(report, '/proc/cpuinfo', 'ProcCpuinfo')
-    attach_file(report, '/proc/cmdline', 'ProcCmdLine')
+    attach_file(report, '/proc/cmdline', 'ProcKernelCmdLine')
     attach_file(report, '/proc/modules', 'ProcModules')
     attach_file(report, '/var/log/udev', 'UdevLog')
 
-    report['Lspci'] = command_output(['lspci','-vvnn'])
+    if os.path.exists('/sys/bus/pci'):
+        report['Lspci'] = command_output(['lspci','-vvnn'])
     report['Lsusb'] = command_output(['lsusb'])
     report['UdevDb'] = command_output(['udevadm', 'info', '--export-db'])
 
@@ -163,10 +181,11 @@ def attach_alsa(report):
     report['PciMultimedia'] = pci_devices(PCI_MULTIMEDIA)
 
     cards = []
-    for line in open('/proc/asound/cards'):
-        if ']:' in line:
-            fields = line.lstrip().split()
-            cards.append(int(fields[0]))
+    if os.path.exists('/proc/asound/cards'):
+        for line in open('/proc/asound/cards'):
+            if ']:' in line:
+                fields = line.lstrip().split()
+                cards.append(int(fields[0]))
 
     for card in cards:
         key = 'Card%d.Amixer.info' % card
@@ -221,7 +240,7 @@ def command_output(command, input = None, stderr = subprocess.STDOUT):
     try:
        sp = subprocess.Popen(command, stdout=subprocess.PIPE,
                              stderr=stderr, close_fds=True, env=env)
-    except OSError, e:
+    except OSError as e:
        return 'Error: ' + str(e)
 
     out = sp.communicate(input)[0]
@@ -231,14 +250,7 @@ def command_output(command, input = None, stderr = subprocess.STDOUT):
        return 'Error: command %s failed with exit code %i: %s' % (
            str(command), sp.returncode, out)
 
-def root_command_output(command, input = None, stderr = subprocess.STDOUT):
-    '''Try to execute given command (array) as root and return its stdout. 
-
-    This passes the command through gksu, kdesudo, or sudo, depending on the
-    running desktop environment.
-    
-    In case of failure, a textual error gets returned.
-    '''
+def _root_command_prefix():
     if os.getuid() == 0:
         prefix = []
     elif os.getenv('DISPLAY') and \
@@ -256,7 +268,60 @@ def root_command_output(command, input = None, stderr = subprocess.STDOUT):
     else:
         prefix = ['sudo']
 
-    return command_output(prefix + command, input, stderr)
+    return prefix
+
+def root_command_output(command, input = None, stderr = subprocess.STDOUT):
+    '''Try to execute given command (array) as root and return its stdout. 
+
+    This passes the command through gksu, kdesudo, or sudo, depending on the
+    running desktop environment.
+    
+    In case of failure, a textual error gets returned.
+    '''
+    assert type(command) == type([]), 'command must be a list'
+    return command_output(_root_command_prefix() + command, input, stderr)
+
+def attach_root_command_outputs(report, command_map):
+    '''Execute multiple commands as root and put their outputs into report.
+
+    command_map is a keyname -> 'shell command' dictionary with the commands to
+    run. They are all run through /bin/sh, so you need to take care of shell
+    escaping yourself. To include stderr output of a command, end it with
+    "2>&1".
+
+    Just like root_command_output() this will use gksu, kdesudo, or sudo for
+    gaining root privileges, depending on the running desktop environment.
+
+    This is preferrable to using root_command_output() multiple times, as that
+    will ask for the password every time.
+    '''
+    workdir = tempfile.mkdtemp()
+    try:
+        # create a shell script with all the commands
+        script_path = os.path.join(workdir, ':script:')
+        script = open(script_path, 'w')
+        for keyname, command in command_map.items():
+            assert hasattr(command, 'strip'), 'command must be a string (shell command)'
+            # use "| cat" here, so that we can end commands with 2>&1
+            # (otherwise it would have the wrong redirection order)
+            script.write('%s | cat > %s\n' % (command, os.path.join(workdir, keyname)))
+        script.close()
+
+        # run script
+        env = os.environ.copy()
+        env['LC_MESSAGES'] = 'C'
+        env['LANGUAGE'] = ''
+        sp = subprocess.Popen(_root_command_prefix() + ['/bin/sh', script_path],
+            close_fds=True, env=env)
+        sp.wait()
+
+        # now read back the individual outputs
+        for keyname in command_map:
+            f = open(os.path.join(workdir, keyname))
+            report[keyname] = f.read()
+            f.close()
+    finally:
+        shutil.rmtree(workdir)
 
 def recent_syslog(pattern):
     '''Extract recent messages from syslog which match a regex.
@@ -264,9 +329,12 @@ def recent_syslog(pattern):
     pattern should be a "re" object.
     '''
     lines = ''
-    for line in open('/var/log/syslog'):
-        if pattern.search(line):
-            lines += line
+    try:
+        for line in open('/var/log/syslog'):
+            if pattern.search(line):
+                lines += line
+    except IOError:
+        return []
     return lines
 
 def xsession_errors(pattern):
@@ -401,7 +469,7 @@ def attach_network(report):
 def attach_wifi(report):
     '''Attach wireless (WiFi) network information to report.'''
 
-    report['WifiSyslog'] = recent_syslog(re.compile(r'(NetworkManager|modem-manager|dhclient|kernel):'))
+    report['WifiSyslog'] = recent_syslog(re.compile(r'(NetworkManager|modem-manager|dhclient|kernel)(\[\d+\])?:'))
     report['IwConfig'] = re.sub('Encryption key:(.*)', 'Encryption key: <hidden>', command_output(['iwconfig']))
     report['RfKill'] = command_output(['rfkill', 'list'])
     report['CRDA'] = command_output(['iw', 'reg', 'get'])
@@ -446,20 +514,30 @@ def package_versions(*packages):
     
     Arguments may be package names or globs, e. g. "foo*"
     '''
-    versions = ''
+    versions = []
     for package_pattern in packages:
         if not package_pattern:
             continue
-        for package in packaging.package_name_glob(package_pattern):
+
+        matching_packages = packaging.package_name_glob(package_pattern)
+
+        if not matching_packages:
+            versions.append((package_pattern, 'N/A'))
+
+        for package in sorted(matching_packages):
             try:
                 version = packaging.get_version(package)
             except ValueError:
                 version = 'N/A'
             if version is None:
                 version = 'N/A'
-            versions += '%s %s\n' % (package, version)
+            versions.append((package,version))
 
-    return versions
+    package_width, version_width = \
+        map(max, [map(len, t) for t in zip(*versions)])
+
+    fmt = '%%-%ds %%s' % package_width
+    return '\n'.join([fmt % v for v in versions])
 
 def shared_libraries(path):
     '''Returns a list of strings containing the sonames of shared libraries
@@ -596,7 +674,7 @@ def attach_drm_info(report):
 
 if __name__ == '__main__':
 
-    import unittest, tempfile
+    import unittest
 
     class _T(unittest.TestCase):
         def test_module_license_evaluation(self):
@@ -622,10 +700,10 @@ if __name__ == '__main__':
             #  - fake BAD module
 
             # direct license check
-            self.assert_('GPL' in _get_module_license('isofs'))
+            self.assertTrue('GPL' in _get_module_license('isofs'))
             self.assertEqual(_get_module_license('does-not-exist'), 'invalid')
-            self.assert_('GPL' in _get_module_license(good_ko.name))
-            self.assert_('BAD' in _get_module_license(bad_ko.name))
+            self.assertTrue('GPL' in _get_module_license(good_ko.name))
+            self.assertTrue('BAD' in _get_module_license(bad_ko.name))
 
             # check via nonfree_kernel_modules logic
             f = tempfile.NamedTemporaryFile()
@@ -633,9 +711,44 @@ if __name__ == '__main__':
                     (good_ko.name,bad_ko.name))
             f.flush()
             nonfree = nonfree_kernel_modules(f.name)
-            self.failIf('isofs' in nonfree)
-            self.assert_('does-not-exist' in nonfree)
-            self.failIf(good_ko.name in nonfree)
-            self.assert_(bad_ko.name in nonfree)
+            self.assertFalse('isofs' in nonfree)
+            self.assertTrue('does-not-exist' in nonfree)
+            self.assertFalse(good_ko.name in nonfree)
+            self.assertTrue(bad_ko.name in nonfree)
+
+        def test_attach_dmesg(self):
+            '''attach_dmesg() does not overwrite already existing data'''
+
+            report = {}
+
+            attach_dmesg(report)
+            self.assertTrue(report['BootDmesg'].startswith('['))
+            self.assertTrue(len(report['BootDmesg']) > 500)
+            self.assertTrue(report['CurrentDmesg'].startswith('['))
+
+        def test_dmesg_overwrite(self):
+            '''attach_dmesg() does not overwrite already existing data'''
+
+            report = {'BootDmesg': 'existingboot'}
+
+            attach_dmesg(report)
+            self.assertEqual(report['BootDmesg'][:50], 'existingboot')
+            self.assertTrue(report['CurrentDmesg'].startswith('['))
+            
+            report = {'BootDmesg': 'existingboot', 'CurrentDmesg': 'existingcurrent' }
+
+            attach_dmesg(report)
+            self.assertEqual(report['BootDmesg'], 'existingboot')
+            self.assertEqual(report['CurrentDmesg'], 'existingcurrent')
+
+        def test_no_crashes(self):
+            '''functions do not crash (very shallow'''
+
+            report = {}
+            attach_hardware(report)
+            attach_alsa(report)
+            attach_network(report)
+            attach_wifi(report)
+            attach_printing(report)
 
     unittest.main()
