@@ -48,7 +48,11 @@ def apport_excepthook(exc_type, exc_obj, exc_tb):
         if not enabled():
             return
 
-        from cStringIO import StringIO
+        try:
+            from cStringIO import StringIO
+        except ImportError:
+            from io import StringIO
+
         import re, tempfile, traceback
         from apport.fileutils import likely_packaged
 
@@ -102,7 +106,7 @@ def apport_excepthook(exc_type, exc_obj, exc_tb):
                 # don't clobber existing report
                 return
         report_file = os.fdopen(os.open(pr_filename,
-            os.O_WRONLY|os.O_CREAT|os.O_EXCL, 0600), 'w')
+            os.O_WRONLY|os.O_CREAT|os.O_EXCL, 0o600), 'w')
         try:
             pr.write(report_file)
         finally:
@@ -154,15 +158,15 @@ def func(x):
 func(42)
 ''' % extracode)
                 os.close(fd)
-                os.chmod(script, 0755)
+                os.chmod(script, 0o755)
 
                 p = subprocess.Popen([script, 'testarg1', 'testarg2'],
                     stdout=subprocess.PIPE, stderr=subprocess.PIPE)
                 err = p.communicate()[1]
                 self.assertEqual(p.returncode, 1,
                     'crashing test python program exits with failure code')
-                self.assert_('Exception: This should happen.' in err)
-                self.failIf('OSError' in err, err)
+                self.assertTrue('Exception: This should happen.' in err)
+                self.assertFalse('OSError' in err, err)
             finally:
                 os.unlink(script)
 
@@ -179,7 +183,7 @@ func(42)
             try:
                 self.assertEqual(len(reports), 1, 'crashed Python program produced a report')
                 self.assertEqual(stat.S_IMODE(os.stat(reports[0]).st_mode),
-                    0600, 'report has correct permissions')
+                    0o600, 'report has correct permissions')
 
                 pr = problem_report.ProblemReport()
                 pr.load(open(reports[0]))
@@ -192,13 +196,13 @@ func(42)
                 'Traceback', 'ProblemType', 'ProcEnviron', 'ProcStatus',
                 'ProcCmdline', 'Date', 'ExecutablePath', 'ProcMaps',
                 'UserGroups']
-            self.assert_(set(expected_keys).issubset(set(pr.keys())),
+            self.assertTrue(set(expected_keys).issubset(set(pr.keys())),
                 'report has necessary fields')
-            self.assert_('bin/python' in pr['InterpreterPath'])
+            self.assertTrue('bin/python' in pr['InterpreterPath'])
             self.assertEqual(pr['ExecutablePath'], script)
             self.assertEqual(pr['PythonArgs'], "['%s', 'testarg1', 'testarg2']" % script)
-            self.assert_(pr['Traceback'].startswith('Traceback'))
-            self.assert_("func\n    raise Exception, 'This should happen." in pr['Traceback'])
+            self.assertTrue(pr['Traceback'].startswith('Traceback'))
+            self.assertTrue("func\n    raise Exception, 'This should happen." in pr['Traceback'])
 
         def test_existing(self):
             '''Python crash hook overwrites seen existing files.'''
@@ -212,7 +216,7 @@ func(42)
                 to_del.update(reports)
                 self.assertEqual(len(reports), 1, 'crashed Python program produced a report')
                 self.assertEqual(stat.S_IMODE(os.stat(reports[0]).st_mode),
-                    0600, 'report has correct permissions')
+                    0o600, 'report has correct permissions')
 
                 # touch report -> "seen" case
                 apport.fileutils.mark_report_seen(reports[0])
@@ -246,7 +250,7 @@ func(42)
             try:
                 self.assertEqual(len(reports), 1, 'crashed Python program produced a report')
                 self.assertEqual(stat.S_IMODE(os.stat(reports[0]).st_mode),
-                    0600, 'report has correct permissions')
+                    0o600, 'report has correct permissions')
 
                 pr = problem_report.ProblemReport()
                 pr.load(open(reports[0]))
@@ -259,10 +263,10 @@ func(42)
                 'Traceback', 'ProblemType', 'ProcEnviron', 'ProcStatus',
                 'ProcCmdline', 'Date', 'ExecutablePath', 'ProcMaps',
                 'UserGroups']
-            self.assert_(set(expected_keys).issubset(set(pr.keys())),
+            self.assertTrue(set(expected_keys).issubset(set(pr.keys())),
                 'report has necessary fields')
-            self.assert_('bin/python' in pr['InterpreterPath'])
-            self.assert_(pr['Traceback'].startswith('Traceback'))
+            self.assertTrue('bin/python' in pr['InterpreterPath'])
+            self.assertTrue(pr['Traceback'].startswith('Traceback'))
 
         def _assert_no_reports(self):
             '''Assert that there are no crash reports.'''
@@ -310,7 +314,7 @@ def func(x):
 func(42)
 ''')
                 os.close(fd)
-                os.chmod(script, 0755)
+                os.chmod(script, 0o755)
 
                 # move aside current ignore file
                 if os.path.exists(ifpath):
@@ -328,7 +332,7 @@ func(42)
                 err = p.communicate()[1]
                 self.assertEqual(p.returncode, 1,
                     'crashing test python program exits with failure code')
-                self.assert_('Exception: This should happen.' in err)
+                self.assertTrue('Exception: This should happen.' in err)
 
             finally:
                 os.unlink(script)
