@@ -18,6 +18,7 @@ import warnings
 warnings.filterwarnings('ignore', 'apt API not stable yet', FutureWarning)
 import apt
 
+import apport
 from apport.packaging import PackageInfo
 
 class __AptDpkgPackageInfo(PackageInfo):
@@ -28,8 +29,6 @@ class __AptDpkgPackageInfo(PackageInfo):
         self._apt_cache = None
         self._contents_dir = None
         self._mirror = None
-        # Checking whether we should use the API from python-apt 0.7.9
-        self.apt_pre_079 = not hasattr(apt.Package, 'installed')
 
         self.configuration = '/etc/default/apport'
 
@@ -63,29 +62,21 @@ class __AptDpkgPackageInfo(PackageInfo):
         try:
             return self._cache()[package]
         except KeyError:
-            raise ValueError, 'package does not exist'
+            raise ValueError('package does not exist')
 
     def get_version(self, package):
         '''Return the installed version of a package.'''
 
         pkg = self._apt_pkg(package)
-        if self.apt_pre_079:
-            if not pkg.isInstalled:
-                raise ValueError, 'package does not exist'
-            return pkg.installedVersion
-        else:
-            inst = pkg.installed
-            if not inst:
-                raise ValueError, 'package does not exist'
-            return inst.version
+        inst = pkg.installed
+        if not inst:
+            raise ValueError('package does not exist')
+        return inst.version
 
     def get_available_version(self, package):
         '''Return the latest available version of a package.'''
 
-        if self.apt_pre_079:
-            return self._apt_pkg(package).candidateVersion
-        else:
-            return self._apt_pkg(package).candidate.version
+        return self._apt_pkg(package).candidate.version
 
     def get_dependencies(self, package):
         '''Return a list of packages a package depends on.'''
@@ -100,15 +91,12 @@ class __AptDpkgPackageInfo(PackageInfo):
     def get_source(self, package):
         '''Return the source package name for a package.'''
 
-        if self.apt_pre_079:
-            return self._apt_pkg(package).sourcePackageName
+        if self._apt_pkg(package).installed:
+            return self._apt_pkg(package).installed.source_name
+        elif self._apt_pkg(package).candidate:
+            return self._apt_pkg(package).candidate.source_name
         else:
-            if self._apt_pkg(package).installed:
-                return self._apt_pkg(package).installed.source_name
-            elif self._apt_pkg(package).candidate:
-                return self._apt_pkg(package).candidate.source_name
-            else:
-                raise ValueError, 'package %s does not exist' % package
+            raise ValueError('package %s does not exist' % package)
 
     def is_distro_package(self, package):
         '''Check if a package is a genuine distro package (True) or comes from
@@ -121,17 +109,13 @@ class __AptDpkgPackageInfo(PackageInfo):
 
         pkg = self._apt_pkg(package)
         # some PPA packages have installed version None, see LP#252734
-        if self.apt_pre_079:
-            if pkg.isInstalled and pkg.installedVersion is None:
-                return False
-        else:
-            if pkg.installed and pkg.installed.version is None:
-                return False
+        if pkg.installed and pkg.installed.version is None:
+            return False
 
         native_origins = [this_os]
         try:
             for f in os.listdir('/etc/apport/native-origins.d'):
-                for line in open(f):
+                for line in open(os.path.join('/etc/apport/native-origins.d', f)):
                     line = line.strip()
                     if line:
                         native_origins.append(line)
@@ -139,10 +123,7 @@ class __AptDpkgPackageInfo(PackageInfo):
             pass
 
         origins = None
-        if self.apt_pre_079:
-            origins = pkg.candidateOrigin
-        else:
-            origins = pkg.candidate.origins
+        origins = pkg.candidate.origins
         if origins: # might be None
             for o in origins:
                 if o.origin in native_origins:
@@ -155,15 +136,12 @@ class __AptDpkgPackageInfo(PackageInfo):
         This might differ on multiarch architectures (e. g.  an i386 Firefox
         package on a x86_64 system)'''
 
-        if self.apt_pre_079:
-            return self._apt_pkg(package).architecture or 'unknown'
+        if self._apt_pkg(package).installed:
+            return self._apt_pkg(package).installed.architecture or 'unknown'
+        elif self._apt_pkg(package).candidate:
+            return self._apt_pkg(package).candidate.architecture or 'unknown'
         else:
-            if self._apt_pkg(package).installed:
-                return self._apt_pkg(package).installed.architecture or 'unknown'
-            elif self._apt_pkg(package).candidate:
-                return self._apt_pkg(package).candidate.architecture or 'unknown'
-            else:
-                raise ValueError, 'package %s does not exist' % package
+            raise ValueError('package %s does not exist' % package)
 
     def get_files(self, package):
         '''Return list of files shipped by a package.'''
@@ -197,11 +175,11 @@ class __AptDpkgPackageInfo(PackageInfo):
             try:
                 # ignore lines with NUL bytes (happens, LP#96050)
                 if '\0' in line:
-                    print >> sys.stderr, 'WARNING:', sumfile, 'contains NUL character, ignoring line'
+                    apport.warning('%s contains NUL character, ignoring line', sumfile)
                     continue
                 words  = line.split()
                 if not words:
-                    print >> sys.stderr, 'WARNING:', sumfile, 'contains empty line, ignoring line'
+                    apport.warning('%s contains empty line, ignoring line', sumfile)
                     continue
                 s = os.stat('/' + words[-1])
                 if max(s.st_mtime, s.st_ctime) <= max_time:
@@ -364,7 +342,7 @@ class __AptDpkgPackageInfo(PackageInfo):
         debug_pkgname = 'linux-image-debug-%s' % kver
         c = self._cache()
         if c.has_key(debug_pkgname) and c[debug_pkgname].isInstalled:
-            #print 'kernel ddeb already installed'
+            #print('kernel ddeb already installed')
             return (installed, outdated)
         target_dir = apt_pkg.Config.FindDir('Dir::Cache::archives')+'/partial'
         deb = '%s_%s_%s.ddeb' % (debug_pkgname, ver, arch)
@@ -414,16 +392,16 @@ class __AptDpkgPackageInfo(PackageInfo):
             else:
                 c.update()
             c.open(apt.progress.OpProgress())
-        except SystemError, e:
+        except SystemError as e:
             if 'Hash Sum mismatch' in str(e):
                 # temporary archive inconsistency
-                print >> sys.stderr, str(e), 'aborting'
+                apport.error('%s, aborting' % str(e))
                 sys.exit(99) # signal crash digger about transient error
             else:
                 raise
         except apt.cache.LockFailedException:
             if os.geteuid() != 0:
-                print >> sys.stderr, 'WARNING: Could not update apt, you need to be root'
+                apport.error('Could not update apt, you need to be root')
             else:
                 raise
 
@@ -439,42 +417,37 @@ class __AptDpkgPackageInfo(PackageInfo):
             try:
                 (pkg, version) = l.split()[:2]
             except ValueError:
-                print >> sys.stderr, 'WARNING: invalid Package/Dependencies line: ', l
+                apport.warning('invalid Package/Dependencies line: %s', l)
                 # invalid line, ignore
                 continue
             dependency_versions[pkg] = version
             try:
                 if self.get_architecture(pkg) != 'all':
-                    dependency_versions[pkg+'-dbgsym'] = dependency_versions[pkg]
+                    if no_pkg and c.has_key(pkg+'-dbg'):
+                        dependency_versions[pkg+'-dbg'] = dependency_versions[pkg]
+                    else:
+                        dependency_versions[pkg+'-dbgsym'] = dependency_versions[pkg]
             except ValueError:
-                print >> sys.stderr, 'WARNING: package %s not known to package cache' % pkg
+                apport.warning('package %s not known to package cache', pkg)
 
         for pkg, ver in dependency_versions.iteritems():
             if not c.has_key(pkg):
-                print >> sys.stderr, 'WARNING: package %s not available' % pkg
+                apport.warning('package %s not available', pkg)
                 continue
 
             # ignore packages which are already installed in the right version
-            if self.apt_pre_079:
-                if (ver and c[pkg].isInstalled and c[pkg].installedVersion ==\
-                    ver) or (not ver and c[pkg].isInstalled):
-                    continue
-            else:
-                if (ver and c[pkg].installed and c[pkg].installed.version ==\
-                    ver) or (not ver and c[pkg].installed):
-                    continue
+            if (ver and c[pkg].installed and c[pkg].installed.version ==\
+                ver) or (not ver and c[pkg].installed):
+                continue
 
             candidate_version = None
-            if self.apt_pre_079:
-                candidate_version = c[pkg].candidateVersion
-            else:
-                candidate_version = c[pkg].candidate.version
+            candidate_version = c[pkg].candidate.version
             if ver and candidate_version != ver:
                 if not pkg.endswith('-dbgsym'):
                     outdated += '%s: installed version %s, latest version: %s\n' % (
                         pkg, ver, candidate_version)
-                print >> sys.stderr, 'WARNING: %s version %s required, but %s is available' % (
-                    pkg, ver, candidate_version)
+                apport.warning('%s version %s required, but %s is available',
+                        pkg, ver, candidate_version)
                 if not unpack_only:
                     uninstallable.append (c[pkg].name)
                     continue
@@ -493,7 +466,7 @@ class __AptDpkgPackageInfo(PackageInfo):
             installProgress = apt.progress.DumbInstallProgress()
 
         try:
-            if c.getChanges():
+            if c.get_changes():
                 os.environ['DEBIAN_FRONTEND'] = 'noninteractive'
                 if unpack_only:
                     self.fetch_unpack(c, fetchProgress, no_pkg, verbosity)
@@ -501,15 +474,14 @@ class __AptDpkgPackageInfo(PackageInfo):
                     try:
                         c.commit(fetchProgress, installProgress)
                     except SystemError:
-                        print >> sys.stderr, 'Error: Could not install all archives. If you use this tool on a production system, it is recommended to use the -u option. See --help for details.'
-                        sys.exit(1)
+                        apport.fatal('Could not install all archives. If you use this tool on a production system, it is recommended to use the -u option. See --help for details.')
 
                 # after commit(), the Cache object does not empty the pending
                 # changes, so we need to reinitialize it to avoid applying the same
                 # changes again below
                 installed = [p.name for p in c.getChanges()]
                 c = apt.Cache()
-        except IOError, e:
+        except IOError as e:
             pass # we will complain to the user later
 
         # package hooks might reassign Package:, check that we have the originally crashing binary
@@ -517,12 +489,12 @@ class __AptDpkgPackageInfo(PackageInfo):
             if path in report and not os.path.exists(report[path]):
                 pkg = self.get_file_package(report[path], True)
                 if pkg:
-                    print 'Installing extra package', pkg, 'to get', path
+                    print('Installing extra package %s to get %s' % (pkg, path))
                     c[pkg].markInstall(False)
                 else:
                     err = 'current version of package %s does not contain the program %s any more' % (report['Package'], path)
                     outdated += err + '\n'
-                    print >> sys.stderr, 'WARNING:', err
+                    apport.warning(err)
 
         # check list of libraries that the crashed process referenced at
         # runtime and warn about those which are not available
@@ -532,7 +504,7 @@ class __AptDpkgPackageInfo(PackageInfo):
                 if not l.strip():
                     continue
                 cols = l.split()
-                if 'x' in cols[1] and len(cols) == 6 and '.so' in cols[5]:
+                if len(cols) == 6 and 'x' in cols[1] and '.so' in cols[5]:
                     lib = os.path.realpath(cols[5])
                     libs.add(lib)
 
@@ -545,19 +517,19 @@ class __AptDpkgPackageInfo(PackageInfo):
             if pkg:
                 if not os.path.exists(l):
                     if pkg in uninstallable:
-                        print >> sys.stderr, 'WARNING: %s cannot be installed (incompatible version)' % pkg
+                        apport.warning('%s cannot be installed (incompatible version)', pkg)
                         continue
                     if c.has_key(pkg):
                         c[pkg].markInstall(False)
                     else:
-                        print >> sys.stderr, 'WARNING: %s was loaded at runtime, but its package %s is not available' % (l, pkg)
+                        apport.warning('%s was loaded at runtime, but its package %s is not available', l, pkg)
 
                 if c.has_key(pkg+'-dbgsym') and pkg+'-dbgsym' not in uninstallable :
                     c[pkg+'-dbgsym'].markInstall(False)
                 else:
-                    print >> sys.stderr, 'WARNING: %s-dbgsym is not available or is incompatible' % pkg
+                    apport.warning('%s-dbgsym is not available or is incompatible', pkg)
             else:
-                    print >> sys.stderr, 'WARNING: %s is needed, but cannot be mapped to a package' % l
+                    apport.warning('%s is needed, but cannot be mapped to a package', l)
 
         try:
             if c.getChanges():
@@ -567,17 +539,12 @@ class __AptDpkgPackageInfo(PackageInfo):
                 else:
                     c.commit(fetchProgress, installProgress)
             installed += [p.name for p in c.getChanges()]
-        except (SystemError, IOError), e:
-            print >> sys.stderr, 'WARNING: could not install missing packages:', e
+        except (SystemError, IOError) as e:
+            apport.warning('could not install missing packages: %s', str(e))
             if os.geteuid() != 0:
-                print >> sys.stderr, 'You either need to call this program as root or install these packages manually:'
+                apport.error('You either need to call this program as root or install these packages manually:')
             for p in c.getChanges():
-                if self.apt_pre_079:
-                    print >> sys.stderr, '  %s %s' % (p.name,
-                                                      p.candidateVersion)
-                else:
-                    print >> sys.stderr, '  %s %s' % (p.name,
-                                                      p.candidate.version)
+                apport.error('  %s %s', p.name, p.candidate.version)
 
         return (installed, outdated)
 
@@ -612,7 +579,7 @@ class __AptDpkgPackageInfo(PackageInfo):
         if dpkg.returncode == 0:
             return out
         else:
-            raise ValueError, 'package does not exist'
+            raise ValueError('package does not exist')
 
     def _check_files_md5(self, sumfile):
         '''Internal function for calling md5sum.
@@ -655,7 +622,7 @@ class __AptDpkgPackageInfo(PackageInfo):
                     self._mirror = fields[1]
                     break
             else:
-                raise SystemError, 'cannot determine default mirror: /etc/apt/sources.list does not contain a valid deb line'
+                raise SystemError('cannot determine default mirror: /etc/apt/sources.list does not contain a valid deb line')
 
         return self._mirror
 
@@ -758,7 +725,7 @@ class __AptDpkgPackageInfo(PackageInfo):
         d = tempfile.mkdtemp()
         d2 = tempfile.mkdtemp()
         try:
-            tar = subprocess.Popen(['tar', '-C', d, '-xz'], stdin=subprocess.PIPE)
+            tar = subprocess.Popen(['tar', '-xz'], cwd=d, stdin=subprocess.PIPE)
             tar.communicate(control_tar)
             assert tar.returncode == 0
             for s in ('preinst', 'postinst', 'prerm', 'postrm'):
@@ -767,7 +734,7 @@ class __AptDpkgPackageInfo(PackageInfo):
                     os.unlink(path)
 
             control_tar_new = os.path.join(d2, 'control.tar.gz')
-            tar = subprocess.Popen(['tar', '-C', d, '-cz', '.'],
+            tar = subprocess.Popen(['tar', '-cz', '.'], cwd=d,
                 stdin=subprocess.PIPE, stdout=open(control_tar_new, 'w'))
             assert tar.wait() == 0
 
@@ -794,8 +761,8 @@ class __AptDpkgPackageInfo(PackageInfo):
         pm = apt.apt_pkg.GetPackageManager(cache._depcache)
         try:
             res = cache._fetchArchives(fetcher, pm)
-        except IOError, e:
-            print >> sys.stderr, 'ERROR: could not fetch all archives:', e
+        except IOError as e:
+            apport.error('could not fetch all archives: %s', str(e))
 
         # extract
         if verbosity:
@@ -805,15 +772,15 @@ class __AptDpkgPackageInfo(PackageInfo):
         if no_dpkg:
             for i in fetcher.Items:
                 if verbosity:
-                    print 'Extracting', i.DestFile
+                    print('Extracting ' + i.DestFile)
                 if subprocess.call(['dpkg', '-x', i.DestFile, '/'], stdout=so,
                     stderr=subprocess.STDOUT) != 0:
-                    print >> sys.stderr, 'WARNING: %s failed to extract' % i.DestFile
+                    apport.warning('%s failed to extract', i.DestFile)
         else:
             res = subprocess.call(['dpkg', '--force-depends', '--force-overwrite', '--unpack'] + 
                 [klass.deb_without_preinst(i.DestFile) for i in fetcher.Items], stdout=so)
             if res != 0:
-                raise IOError, 'dpkg failed to unpack archives'
+                raise IOError('dpkg failed to unpack archives')
 
         # remove other maintainer scripts
         for c in cache.getChanges():
@@ -874,14 +841,14 @@ if __name__ == '__main__':
         def test_get_version(self):
             '''get_version().'''
 
-            self.assert_(impl.get_version('libc6').startswith('2'))
+            self.assertTrue(impl.get_version('libc6').startswith('2'))
             self.assertRaises(ValueError, impl.get_version, 'nonexisting')
             self.assertRaises(ValueError, impl.get_version, 'wukrainian')
 
         def test_get_available_version(self):
             '''get_available_version().'''
 
-            self.assert_(impl.get_available_version('libc6').startswith('2'))
+            self.assertTrue(impl.get_available_version('libc6').startswith('2'))
             self.assertRaises(ValueError, impl.get_available_version, 'nonexisting')
 
         def test_get_dependencies(self):
@@ -889,36 +856,36 @@ if __name__ == '__main__':
 
             # package with both Depends: and Pre-Depends:
             d  = impl.get_dependencies('bash')
-            self.assert_(len(d) > 2)
-            self.assert_('libc6' in d)
+            self.assertTrue(len(d) > 2)
+            self.assertTrue('libc6' in d)
             for dep in d:
-                self.assert_(impl.get_version(dep))
+                self.assertTrue(impl.get_version(dep))
 
             # Pre-Depends: only
             d  = impl.get_dependencies('coreutils')
-            self.assert_(len(d) >= 1)
-            self.assert_('libc6' in d)
+            self.assertTrue(len(d) >= 1)
+            self.assertTrue('libc6' in d)
             for dep in d:
-                self.assert_(impl.get_version(dep))
+                self.assertTrue(impl.get_version(dep))
 
             # Depends: only
             d  = impl.get_dependencies('libc6')
-            self.assert_(len(d) >= 1)
+            self.assertTrue(len(d) >= 1)
             for dep in d:
-                self.assert_(impl.get_version(dep))
+                self.assertTrue(impl.get_version(dep))
 
         def test_get_source(self):
             '''get_source().'''
 
             self.assertRaises(ValueError, impl.get_source, 'nonexisting')
             self.assertEqual(impl.get_source('bash'), 'bash')
-            self.assert_('glibc' in impl.get_source('libc6'))
+            self.assertTrue('glibc' in impl.get_source('libc6'))
 
         def test_is_distro_package(self):
             '''is_distro_package().'''
 
             self.assertRaises(ValueError, impl.is_distro_package, 'nonexisting')
-            self.assert_(impl.is_distro_package('bash'))
+            self.assertTrue(impl.is_distro_package('bash'))
             # no False test here, hard to come up with a generic one
 
         def test_get_architecture(self):
@@ -936,7 +903,7 @@ if __name__ == '__main__':
             '''get_files().'''
 
             self.assertRaises(ValueError, impl.get_files, 'nonexisting')
-            self.assert_('/bin/bash' in impl.get_files('bash'))
+            self.assertTrue('/bin/bash' in impl.get_files('bash'))
 
         def test_get_file_package(self):
             '''get_file_package() on installed files.'''
@@ -959,14 +926,16 @@ if __name__ == '__main__':
             try:
                 mapdir = os.path.join(basedir, 'dists', release_name)
                 os.makedirs(mapdir)
-                print >> gzip.open(os.path.join(mapdir, 'Contents-%s.gz' %
-                    impl.get_system_architecture()), 'w'), '''
+                f = gzip.open(os.path.join(mapdir, 'Contents-%s.gz' %
+                    impl.get_system_architecture()), 'w')
+                f.write('''
  foo header
 FILE                                                    LOCATION
 usr/bin/frobnicate                                      foo/frob
 usr/bin/frob                                            foo/frob-utils
 bo/gu/s                                                 na/mypackage
-'''
+''')
+                f.close()
 
                 self.assertEqual(impl.get_file_package('usr/bin/frob', False, mapdir), None)
                 # must not match frob (same file name prefix)
@@ -987,7 +956,7 @@ bo/gu/s                                                 na/mypackage
                 os.mkdir(cache_dir)
                 self.assertEqual(impl.get_file_package('usr/bin/frob', True, cache_dir), 'frob-utils')
                 self.assertEqual(len(os.listdir(cache_dir)), 1)
-                self.assert_(os.listdir(cache_dir)[0].startswith('Contents-'))
+                self.assertTrue(os.listdir(cache_dir)[0].startswith('Contents-'))
                 self.assertEqual(impl.get_file_package('/bo/gu/s', True, cache_dir), 'mypackage')
             finally:
                 shutil.rmtree(basedir)
@@ -1013,7 +982,7 @@ bo/gu/s                                                 na/mypackage
             arch = impl.get_system_architecture()
             # must be nonempty without line breaks
             self.assertNotEqual(arch, '')
-            self.assert_('\n' not in arch)
+            self.assertTrue('\n' not in arch)
 
         def test_compare_versions(self):
             '''compare_versions.'''
@@ -1055,13 +1024,13 @@ bo/gu/s                                                 na/mypackage
         def test_get_kernel_pacakge(self):
             '''get_kernel_package().'''
 
-            self.assert_('linux' in impl.get_kernel_package())
+            self.assertTrue('linux' in impl.get_kernel_package())
 
         def test_package_name_glob(self):
             '''package_name_glob().'''
 
-            self.assert_(len(impl.package_name_glob('a*')) > 5)
-            self.assert_('bash' in impl.package_name_glob('ba*h'))
+            self.assertTrue(len(impl.package_name_glob('a*')) > 5)
+            self.assertTrue('bash' in impl.package_name_glob('ba*h'))
             self.assertEqual(impl.package_name_glob('bash'), ['bash'])
             self.assertEqual(impl.package_name_glob('xzywef*'), [])
 
