@@ -12,11 +12,12 @@
 # option) any later version.  See http://www.gnu.org/copyleft/gpl.html for
 # the full text of the license.
 
-import sys, re
+import sys, re, logging
 
 class ParseSegv(object):
     def __init__(self, registers, disassembly, maps, debug=False):
-        self.debug = debug
+        if debug:
+            logging.basicConfig(level=logging.DEBUG)
 
         self.regs = self.parse_regs(registers)
         self.sp = None
@@ -43,7 +44,7 @@ class ParseSegv(object):
             try:
                 span, perms, bits, dev = items[0:4]
             except:
-                raise ValueError, 'Cannot parse maps line: %s' % (line.strip())
+                raise ValueError('Cannot parse maps line: %s' % (line.strip()))
             if len(items)==5:
                 name = None
             else:
@@ -52,8 +53,7 @@ class ParseSegv(object):
             if name == '[stack]':
                 self.stack_vma = len(maps)
             maps.append({'start': start, 'end': end, 'perms': perms, 'name': name})
-            if self.debug:
-                print >>sys.stderr, start, end, perms, name
+            logging.debug(start, end, perms, name)
         return maps
 
     def parse_regs(self, reg_str):
@@ -61,33 +61,30 @@ class ParseSegv(object):
         for line in reg_str.splitlines():
             reg, hexvalue = line.split()[0:2]
             regs[reg] = int(hexvalue,16)
-            if self.debug:
-                print >>sys.stderr, '%s:0x%08x' % (reg, regs[reg])
+            logging.debug('%s:0x%08x', reg, regs[reg])
         return regs
 
     def parse_disassembly(self, disassembly):
         if not self.regs:
-            raise ValueError, 'Registers not loaded yet!?'
+            raise ValueError('Registers not loaded yet!?')
         lines = disassembly.splitlines()
         # Throw away possible 'Dump' gdb report line
         if len(lines)>0 and lines[0].startswith('Dump'):
             lines.pop(0)
         if len(lines)<1:
-            raise ValueError, 'Failed to load empty disassembly'
+            raise ValueError('Failed to load empty disassembly')
         line = lines[0].strip()
         # Drop GDB 7.1's leading $pc mark
         if line.startswith('=>'):
             line = line[2:].strip()
-        if self.debug:
-            print >>sys.stderr, line
+        logging.debug(line)
         pc_str = line.split()[0]
         if pc_str.startswith('0x'):
             pc = int(pc_str.split(':')[0],16)
         else:
             # Could not identify this instruction line
-            raise ValueError, 'Could not parse PC "%s" from disassembly line: %s' % (pc_str, line)
-        if self.debug:
-            print >>sys.stderr, 'pc: 0x%08x' % (pc)
+            raise ValueError('Could not parse PC "%s" from disassembly line: %s' % (pc_str, line))
+        logging.debug('pc: 0x%08x', pc)
 
         full_insn_str = line.split(':',1)[1].strip()
         # Handle invalid memory
@@ -108,8 +105,7 @@ class ParseSegv(object):
             args_str = insn_parts.pop(-1)
         # Assume remainder is the insn itself
         insn = ' '.join(insn_parts)
-        if self.debug:
-            print >>sys.stderr, 'insn: %s' % (insn)
+        logging.debug('insn: %s', insn)
 
         args = []
         src = None
@@ -118,20 +114,17 @@ class ParseSegv(object):
             # Could not find insn args
             args = None
         else:
-            if self.debug:
-                print >>sys.stderr, 'args: "%s"' % (args_str)
+            logging.debug('args: "%s"', args_str)
 
             for m in re.finditer('([^,\(]*(\(:?[^\)]+\))*)',args_str):
                 if len(m.group(0)):
                     args.append(m.group(0))
             if len(args)>0:
                 src = args[0]
-                if self.debug:
-                    print >>sys.stderr, 'src: %s' % (src)
+                logging.debug('src: %s', src)
             if len(args)>1:
                 dest = args[1]
-                if self.debug:
-                    print >>sys.stderr, 'dest: %s' % (dest)
+                logging.debug('dest: %s', dest)
 
         # Set up possible implicit memory destinations (stack actions)
         if insn in ['push','pop','pushl','popl','call','callq','ret','retq']:
@@ -198,7 +191,7 @@ class ParseSegv(object):
         if reg in self.regs:
             #print 'got %s (%d & %d == %d)' % (reg, self.regs[reg], mask, self.regs[reg] & ~mask)
             return self.regs[reg] & ~mask
-        raise ValueError, "Could not resolve register '%s'" % (reg_orig)
+        raise ValueError("Could not resolve register '%s'" % (reg_orig))
 
     def calculate_arg(self, arg):
         # Check for and pre-remove segment offset
@@ -225,7 +218,7 @@ class ParseSegv(object):
                 add = self.regs[offset[1:]]
             else:
                 if not offset.startswith('0x'):
-                    raise ValueError, 'Unknown offset literal: %s' % (parts[0])
+                    raise ValueError('Unknown offset literal: %s' % (parts[0]))
                 add = int(offset[2:],16) * sign
         else:
             add = 0
@@ -356,24 +349,22 @@ def add_info(report):
         if understood:
             report['SegvReason'] = reason
         report['SegvAnalysis'] = details
-    except BaseException, e:
+    except BaseException as e:
         report['SegvAnalysis'] = 'Failure: %s' % (str(e))
 
 
 if __name__ == '__main__':
     if len(sys.argv)==2 and sys.argv[1] in ['-h','--help']:
-        print 'To run self-test, run without any arguments (or with -v)'
-        print 'To do stand-alone crash parsing:'
-        print '  Usage: %s Registers.txt Disassembly.txt ProcMaps.txt' % (sys.argv[0])
+        print('To run self-test, run without any arguments (or with -v)')
+        print('To do stand-alone crash parsing:')
+        print('  Usage: %s Registers.txt Disassembly.txt ProcMaps.txt' % (sys.argv[0]))
         sys.exit(0)
     elif len(sys.argv)==4:
         segv = ParseSegv(file(sys.argv[1]).read(), \
                          file(sys.argv[2]).read(), \
                          file(sys.argv[3]).read())
         understood, reason, details = segv.report()
-        print reason
-        print ''
-        print details
+        print ('%s\n\n%s' % (reason, details))
         rc = 0
         if not understood:
             rc = 1
@@ -511,7 +502,7 @@ bfc57000-bfc6c000 rw-p 00000000 00:00 0          [stack]
                 self.assertRaises(ValueError, ParseSegv, regs, '', '')
                 try:
                     segv = ParseSegv(regs, '', '')
-                except ValueError, e:
+                except ValueError as e:
                     self.assertTrue('invalid literal for int()' in str(e), str(e))
 
                 regs = 'a 0x10'
