@@ -11,12 +11,21 @@
 # option) any later version.  See http://www.gnu.org/copyleft/gpl.html for
 # the full text of the license.
 
-import zlib, base64, time, UserDict, sys, gzip, struct
-from cStringIO import StringIO
-from email.Encoders import encode_base64
-from email.MIMEMultipart import MIMEMultipart
-from email.MIMEBase import MIMEBase
-from email.MIMEText import MIMEText
+import zlib, base64, time, sys, gzip, struct
+from email.encoders import encode_base64
+from email.mime.multipart import MIMEMultipart
+from email.mime.base import MIMEBase
+from email.mime.text import MIMEText
+try:
+    from cStringIO import StringIO
+except ImportError:
+    from io import StringIO
+
+try:
+    from collections import UserDict
+except ImportError:
+    # Python 2
+    from UserDict import IterableUserDict as UserDict
 
 class CompressedValue:
     '''Represent a ProblemReport value which is gzip compressed.'''
@@ -81,7 +90,7 @@ class CompressedValue:
 
         return self.get_value().splitlines()
 
-class ProblemReport(UserDict.IterableUserDict):
+class ProblemReport(UserDict):
     def __init__(self, type = 'Crash', date = None):
         '''Initialize a fresh problem report.
 
@@ -247,23 +256,24 @@ class ProblemReport(UserDict.IterableUserDict):
                     v = open(v[0]).read() # file name
 
                 if fail_on_empty and len(v) == 0:
-                    raise IOError, 'did not get any data for field ' + k
+                    raise IOError('did not get any data for field ' + k)
 
                 if limit != None and len(v) > limit:
                     del self.data[k]
                     continue
 
-            if type(v) == type(u''):
-                # unicode → str
-                v = v.encode('UTF-8')
+            if sys.version.startswith('2'):
+                if isinstance(v, unicode):
+                    # unicode → str
+                    v = v.encode('UTF-8')
 
             if '\n' in v:
                 # multiline value
-                print >> file, k + ':'
-                print >> file, '', v.replace('\n', '\n ')
+                file.write('%s:\n' % k)
+                file.write(' %s\n' % v.replace('\n', '\n '))
             else:
                 # single line value
-                print >> file, k + ':', v
+                file.write('%s: %s\n' % (k, v))
 
         # now write the binary keys with gzip compression and base64 encoding
         for k in binkeys:
@@ -327,15 +337,15 @@ class ProblemReport(UserDict.IterableUserDict):
 
                 if len(v) >= 4 and v[3]:
                     if size == 0:
-                        raise IOError, 'did not get any data for field %s from %s' % (k, str(v[0]))
+                        raise IOError('did not get any data for field %s from %s' % (k, str(v[0])))
 
             # flush compressor and write the rest
             if not limit or size <= limit:
                 block = bc.flush()
                 # append gzip trailer: crc (32 bit) and size (32 bit)
                 if crc:
-                    block += struct.pack("<L", crc & 0xFFFFFFFFL)
-                    block += struct.pack("<L", size & 0xFFFFFFFFL)
+                    block += struct.pack("<L", crc & 0xFFFFFFFF)
+                    block += struct.pack("<L", size & 0xFFFFFFFF)
 
                 file.write(base64.b64encode(block))
                 file.write('\n')
@@ -383,8 +393,7 @@ class ProblemReport(UserDict.IterableUserDict):
         priority_fields is a set/list specifying the order in which keys should
         appear in the destination file.
         '''
-        keys = self.data.keys()
-        keys.sort()
+        keys = sorted(self.data.keys())
 
         text = ''
         attachments = []
@@ -458,7 +467,7 @@ class ProblemReport(UserDict.IterableUserDict):
                 if type(v) == type(''):
                     v = v.decode('UTF-8', 'replace')
                 # convert unicode to UTF-8 str
-                assert type(v) == type(u'')
+                assert isinstance(v, unicode)
                 v = v.encode('UTF-8')
 
                 lines = len(v.splitlines())
@@ -482,12 +491,13 @@ class ProblemReport(UserDict.IterableUserDict):
         attachments.insert(0, att)
 
         msg = MIMEMultipart()
-        for k, v in extra_headers.iteritems():
+        for k, v in extra_headers.items():
             msg.add_header(k, v)
         for a in attachments:
             msg.attach(a)
 
-        print >> file, msg.as_string()
+        file.write(msg.as_string())
+        file.write('\n')
 
     def __setitem__(self, k, v):
         assert hasattr(k, 'isalnum')
@@ -549,7 +559,7 @@ class _T(unittest.TestCase):
         self.assertEqual(pr['foo'], 'bar')
         self.assertEqual(pr['bar'], ' foo   bar\nbaz\n   blip  ')
         self.assertEqual(pr['ProblemType'], 'Crash')
-        self.assert_(time.strptime(pr['Date']))
+        self.assertTrue(time.strptime(pr['Date']))
         self.assertEqual(pr['dash-key'], '1')
         self.assertEqual(pr['dot.key'], '1')
         self.assertEqual(pr['underscore_key'], '1')
@@ -584,8 +594,8 @@ class _T(unittest.TestCase):
         pr['Bin'].set_value('AB' * 10 + '\0' * 10 + 'Z')
         pr['Large'] = CompressedValue(large_val)
 
-        self.assert_(isinstance(pr['Foo'], CompressedValue))
-        self.assert_(isinstance(pr['Bin'], CompressedValue))
+        self.assertTrue(isinstance(pr['Foo'], CompressedValue))
+        self.assertTrue(isinstance(pr['Bin'], CompressedValue))
         self.assertEqual(pr['Foo'].get_value(), 'FooFoo!')
         self.assertEqual(pr['Bin'].get_value(), 'AB' * 10 + '\0' * 10 + 'Z')
         self.assertEqual(pr['Large'].get_value(), large_val)
@@ -619,14 +629,20 @@ class _T(unittest.TestCase):
 
         pr = ProblemReport(date = 'now!')
         pr['Simple'] = 'bar'
-        pr['SimpleUTF8'] = '1äö2Φ3'
-        pr['SimpleUnicode'] = u'1äö2Φ3'
-        pr['TwoLineUTF8'] = 'pi-π\nnu-η'
-        pr['TwoLineUnicode'] = u'pi-π\nnu-η'
+        if sys.version.startswith('2'):
+            pr['SimpleUTF8'] = '1äö2Φ3'
+            pr['SimpleUnicode'] = '1äö2Φ3'.decode('UTF-8')
+            pr['TwoLineUnicode'] = 'pi-π\nnu-η'.decode('UTF-8')
+            pr['TwoLineUTF8'] = 'pi-π\nnu-η'.decode('UTF-8')
+        else:
+            pr['SimpleUTF8'] = '1äö2Φ3'.encode('UTF-8')
+            pr['SimpleUnicode'] = '1äö2Φ3'
+            pr['TwoLineUnicode'] = 'pi-π\nnu-η'
+            pr['TwoLineUTF8'] = 'pi-π\nnu-η'
         pr['WhiteSpace'] = ' foo   bar\nbaz\n  blip  \n\nafteremptyline'
         io = StringIO()
         pr.write(io)
-        self.assertEqual(io.getvalue(), 
+        self.assertEqual(io.getvalue(),
 '''ProblemType: Crash
 Date: now!
 Simple: bar
@@ -762,7 +778,7 @@ Last: foo
 
         # test that load() cleans up properly
         pr.load(StringIO('ProblemType: Crash'))
-        self.assertEqual(pr.keys(), ['ProblemType'])
+        self.assertEqual(list(pr.keys()), ['ProblemType'])
 
     def test_write_file(self):
         '''writing a report with binary file data.'''
@@ -879,7 +895,7 @@ File: base64
 
         pr2 = ProblemReport()
         pr2.load(io)
-        self.assert_(pr2['BinFile'].endswith('abhello world'))
+        self.assertTrue(pr2['BinFile'].endswith('abhello world'))
         self.assertEqual(len(pr2['BinFile']), 1048576 + len('hello world'))
 
     def test_read_file(self):
@@ -908,7 +924,7 @@ Foo: Bar
         pr.load(StringIO(bin_report), binary='compressed')
         self.assertEqual(pr['Foo'], 'Bar')
         self.assertEqual(pr.has_removed_fields(), False)
-        self.assert_(isinstance(pr['File'], CompressedValue))
+        self.assertTrue(isinstance(pr['File'], CompressedValue))
 
         self.assertEqual(pr['File'].get_value(), 'AB' * 10 + '\0' * 10 + 'Z')
 
@@ -970,14 +986,14 @@ Foo: Bar
         pr = ProblemReport()
         pr.load(io)
 
-        self.assert_(pr['File'] == data)
+        self.assertTrue(pr['File'] == data)
         self.assertEqual(pr['Before'], 'xtestx')
         self.assertEqual(pr['ZAfter'], 'ytesty')
 
         # write it again
         io2 = StringIO()
         pr.write(io2)
-        self.assert_(io.getvalue() == io2.getvalue())
+        self.assertTrue(io.getvalue() == io2.getvalue())
 
         # check gzip compatibility
         io.seek(0)
@@ -1012,11 +1028,11 @@ Foo: Bar
         pr = ProblemReport()
         pr.load(io)
 
-        self.failIf(pr.has_key('FileSmallLimit'))
-        self.failIf(pr.has_key('FileLimitMinus1'))
-        self.assert_(pr['FileExactLimit'] == data)
-        self.assert_(pr['FileLimitPlus1'] == data)
-        self.assert_(pr['FileLimitNone'] == data)
+        self.assertFalse(pr.has_key('FileSmallLimit'))
+        self.assertFalse(pr.has_key('FileLimitMinus1'))
+        self.assertTrue(pr['FileExactLimit'] == data)
+        self.assertTrue(pr['FileLimitPlus1'] == data)
+        self.assertTrue(pr['FileLimitNone'] == data)
         self.assertEqual(pr['Before'], 'xtestx')
         self.assertEqual(pr['ZAfter'], 'ytesty')
 
@@ -1155,12 +1171,18 @@ File: base64
 
         pr = ProblemReport(date = 'now!')
         pr['Simple'] = 'bar'
-        pr['SimpleUTF8'] = '1äö2Φ3'
-        pr['SimpleUnicode'] = u'1äö2Φ3'
+        if sys.version.startswith('2'):
+            pr['SimpleUTF8'] = '1äö2Φ3'
+            pr['SimpleUnicode'] = '1äö2Φ3'.decode('UTF-8')
+            pr['TwoLineUnicode'] = 'pi-π\nnu-η\n'.decode('UTF-8')
+            pr['TwoLineUTF8'] = 'pi-π\nnu-η\n'.decode('UTF-8')
+        else:
+            pr['SimpleUTF8'] = '1äö2Φ3'.encode('UTF-8')
+            pr['SimpleUnicode'] = '1äö2Φ3'
+            pr['TwoLineUnicode'] = 'pi-π\nnu-η\n'
+            pr['TwoLineUTF8'] = 'pi-π\nnu-η\n'
         pr['SimpleLineEnd'] = 'bar\n'
         pr['TwoLine'] = 'first\nsecond\n'
-        pr['TwoLineUTF8'] = 'pi-π\nnu-η\n'
-        pr['TwoLineUnicode'] = u'pi-π\nnu-η\n'
         pr['InlineMargin'] = 'first\nsecond\nthird\nfourth\nfifth\n'
         pr['Multiline'] = ' foo   bar\nbaz\n  blip  \nline4\nline♥5!!\nłıµ€ ⅝\n'
         io = StringIO()
@@ -1168,20 +1190,19 @@ File: base64
         io.seek(0)
 
         msg = email.message_from_file(io)
-        msg_iter = msg.walk()
+        parts = [p for p in msg.walk()]
+        self.assertEqual(len(parts), 3)
 
         # first part is the multipart container
-        part = msg_iter.next()
-        self.assert_(part.is_multipart())
+        self.assertTrue(parts[0].is_multipart())
 
         # second part should be an inline text/plain attachments with all short
         # fields
-        part = msg_iter.next()
-        self.assert_(not part.is_multipart())
-        self.assertEqual(part.get_content_type(), 'text/plain')
-        self.assertEqual(part.get_content_charset(), 'utf-8')
-        self.assertEqual(part.get_filename(), None)
-        self.assertEqual(part.get_payload(decode=True), '''ProblemType: Crash
+        self.assertTrue(not parts[1].is_multipart())
+        self.assertEqual(parts[1].get_content_type(), 'text/plain')
+        self.assertEqual(parts[1].get_content_charset(), 'utf-8')
+        self.assertEqual(parts[1].get_filename(), None)
+        self.assertEqual(parts[1].get_payload(decode=True), '''ProblemType: Crash
 Date: now!
 InlineMargin:
  first
@@ -1205,12 +1226,11 @@ TwoLineUnicode:
 ''')
 
         # third part should be the Multiline: field as attachment
-        part = msg_iter.next()
-        self.assert_(not part.is_multipart())
-        self.assertEqual(part.get_content_type(), 'text/plain')
-        self.assertEqual(part.get_content_charset(), 'utf-8')
-        self.assertEqual(part.get_filename(), 'Multiline.txt')
-        self.assertEqual(part.get_payload(decode=True), ''' foo   bar
+        self.assertTrue(not parts[2].is_multipart())
+        self.assertEqual(parts[2].get_content_type(), 'text/plain')
+        self.assertEqual(parts[2].get_content_charset(), 'utf-8')
+        self.assertEqual(parts[2].get_filename(), 'Multiline.txt')
+        self.assertEqual(parts[2].get_payload(decode=True), ''' foo   bar
 baz
   blip  
 line4
@@ -1218,9 +1238,6 @@ line♥5!!
 łıµ€ ⅝
 ''')
 
-        # no more parts
-        self.assertRaises(StopIteration, msg_iter.next)
-
     def test_write_mime_binary(self):
         '''write_mime() for binary values and file references.'''
 
@@ -1248,78 +1265,69 @@ line♥5!!
         io.seek(0)
 
         msg = email.message_from_file(io)
-        msg_iter = msg.walk()
+        parts = [p for p in msg.walk()]
+        self.assertEqual(len(parts), 7)
 
         # first part is the multipart container
-        part = msg_iter.next()
-        self.assert_(part.is_multipart())
+        self.assertTrue(parts[0].is_multipart())
 
         # second part should be an inline text/plain attachments with all short
         # fields
-        part = msg_iter.next()
-        self.assert_(not part.is_multipart())
-        self.assertEqual(part.get_content_type(), 'text/plain')
-        self.assertEqual(part.get_content_charset(), 'utf-8')
-        self.assertEqual(part.get_filename(), None)
-        self.assertEqual(part.get_payload(decode=True),
+        self.assertTrue(not parts[1].is_multipart())
+        self.assertEqual(parts[1].get_content_type(), 'text/plain')
+        self.assertEqual(parts[1].get_content_charset(), 'utf-8')
+        self.assertEqual(parts[1].get_filename(), None)
+        self.assertEqual(parts[1].get_payload(decode=True),
             'ProblemType: Crash\nContext: Test suite\nDate: now!\n')
 
         # third part should be the File1: file contents as gzip'ed attachment
-        part = msg_iter.next()
-        self.assert_(not part.is_multipart())
-        self.assertEqual(part.get_content_type(), 'application/x-gzip')
-        self.assertEqual(part.get_filename(), 'File1.gz')
+        self.assertTrue(not parts[2].is_multipart())
+        self.assertEqual(parts[2].get_content_type(), 'application/x-gzip')
+        self.assertEqual(parts[2].get_filename(), 'File1.gz')
         f = tempfile.TemporaryFile()
-        f.write(part.get_payload(decode=True))
+        f.write(parts[2].get_payload(decode=True))
         f.seek(0)
         self.assertEqual(gzip.GzipFile(mode='rb', fileobj=f).read(), bin_value)
 
         # fourth part should be the File1.gz: file contents as gzip'ed
         # attachment; write_mime() should not compress it again
-        part = msg_iter.next()
-        self.assert_(not part.is_multipart())
-        self.assertEqual(part.get_content_type(), 'application/x-gzip')
-        self.assertEqual(part.get_filename(), 'File1.gz')
+        self.assertTrue(not parts[3].is_multipart())
+        self.assertEqual(parts[3].get_content_type(), 'application/x-gzip')
+        self.assertEqual(parts[3].get_filename(), 'File1.gz')
         f = tempfile.TemporaryFile()
-        f.write(part.get_payload(decode=True))
+        f.write(parts[3].get_payload(decode=True))
         f.seek(0)
         self.assertEqual(gzip.GzipFile(mode='rb', fileobj=f).read(), bin_value)
 
         # fifth part should be the Value1: value as gzip'ed attachment
-        part = msg_iter.next()
-        self.assert_(not part.is_multipart())
-        self.assertEqual(part.get_content_type(), 'application/x-gzip')
-        self.assertEqual(part.get_filename(), 'Value1.gz')
+        self.assertTrue(not parts[4].is_multipart())
+        self.assertEqual(parts[4].get_content_type(), 'application/x-gzip')
+        self.assertEqual(parts[4].get_filename(), 'Value1.gz')
         f = tempfile.TemporaryFile()
-        f.write(part.get_payload(decode=True))
+        f.write(parts[4].get_payload(decode=True))
         f.seek(0)
         self.assertEqual(gzip.GzipFile(mode='rb', fileobj=f).read(), bin_value)
 
         # sixth part should be the Value1: value as gzip'ed attachment;
         # write_mime should not compress it again
-        part = msg_iter.next()
-        self.assert_(not part.is_multipart())
-        self.assertEqual(part.get_content_type(), 'application/x-gzip')
-        self.assertEqual(part.get_filename(), 'Value1.gz')
+        self.assertTrue(not parts[5].is_multipart())
+        self.assertEqual(parts[5].get_content_type(), 'application/x-gzip')
+        self.assertEqual(parts[5].get_filename(), 'Value1.gz')
         f = tempfile.TemporaryFile()
-        f.write(part.get_payload(decode=True))
+        f.write(parts[5].get_payload(decode=True))
         f.seek(0)
         self.assertEqual(gzip.GzipFile(mode='rb', fileobj=f).read(), bin_value)
 
         # seventh part should be the ZValue: value as gzip'ed attachment;
         # write_mime should not compress it again
-        part = msg_iter.next()
-        self.assert_(not part.is_multipart())
-        self.assertEqual(part.get_content_type(), 'application/x-gzip')
-        self.assertEqual(part.get_filename(), 'ZValue.gz')
+        self.assertTrue(not parts[6].is_multipart())
+        self.assertEqual(parts[6].get_content_type(), 'application/x-gzip')
+        self.assertEqual(parts[6].get_filename(), 'ZValue.gz')
         f = tempfile.TemporaryFile()
-        f.write(part.get_payload(decode=True))
+        f.write(parts[6].get_payload(decode=True))
         f.seek(0)
         self.assertEqual(gzip.GzipFile(mode='rb', fileobj=f).read(), bin_value)
 
-        # no more parts
-        self.assertRaises(StopIteration, msg_iter.next)
-
     def test_write_mime_extra_headers(self):
         '''write_mime() with extra headers.'''
 
@@ -1334,21 +1342,17 @@ line♥5!!
         msg = email.message_from_file(io)
         self.assertEqual(msg['Greeting'], 'hello world')
         self.assertEqual(msg['Foo'], 'Bar')
-        msg_iter = msg.walk()
+        parts = [p for p in msg.walk()]
+        self.assertEqual(len(parts), 2)
 
         # first part is the multipart container
-        part = msg_iter.next()
-        self.assert_(part.is_multipart())
+        self.assertTrue(parts[0].is_multipart())
 
         # second part should be an inline text/plain attachments with all short
         # fields
-        part = msg_iter.next()
-        self.assert_(not part.is_multipart())
-        self.assertEqual(part.get_content_type(), 'text/plain')
-        self.assert_('Simple: bar' in part.get_payload(decode=True))
-
-        # no more parts
-        self.assertRaises(StopIteration, msg_iter.next)
+        self.assertTrue(not parts[1].is_multipart())
+        self.assertEqual(parts[1].get_content_type(), 'text/plain')
+        self.assertTrue('Simple: bar' in parts[1].get_payload(decode=True))
 
     def test_write_mime_filter(self):
         '''write_mime() with key filters.'''
@@ -1365,35 +1369,30 @@ line♥5!!
         io.seek(0)
 
         msg = email.message_from_file(io)
-        msg_iter = msg.walk()
+        parts = [p for p in msg.walk()]
+        self.assertEqual(len(parts), 3)
 
         # first part is the multipart container
-        part = msg_iter.next()
-        self.assert_(part.is_multipart())
+        self.assertTrue(parts[0].is_multipart())
 
         # second part should be an inline text/plain attachments with all short
         # fields
-        part = msg_iter.next()
-        self.assert_(not part.is_multipart())
-        self.assertEqual(part.get_content_type(), 'text/plain')
-        self.assertEqual(part.get_content_charset(), 'utf-8')
-        self.assertEqual(part.get_filename(), None)
-        self.assertEqual(part.get_payload(decode=True), '''ProblemType: Crash
+        self.assertTrue(not parts[1].is_multipart())
+        self.assertEqual(parts[1].get_content_type(), 'text/plain')
+        self.assertEqual(parts[1].get_content_charset(), 'utf-8')
+        self.assertEqual(parts[1].get_filename(), None)
+        self.assertEqual(parts[1].get_payload(decode=True), '''ProblemType: Crash
 Date: now!
 GoodText: Hi
 ''')
 
         # third part should be the GoodBin: field as attachment
-        part = msg_iter.next()
-        self.assert_(not part.is_multipart())
+        self.assertTrue(not parts[2].is_multipart())
         f = tempfile.TemporaryFile()
-        f.write(part.get_payload(decode=True))
+        f.write(parts[2].get_payload(decode=True))
         f.seek(0)
         self.assertEqual(gzip.GzipFile(mode='rb', fileobj=f).read(), bin_value)
 
-        # no more parts
-        self.assertRaises(StopIteration, msg_iter.next)
-
     def test_write_mime_order(self):
         '''write_mime() with keys ordered.'''
 
@@ -1410,20 +1409,19 @@ GoodText: Hi
         io.seek(0)
 
         msg = email.message_from_file(io)
-        msg_iter = msg.walk()
+        parts = [p for p in msg.walk()]
+        self.assertEqual(len(parts), 2)
 
         # first part is the multipart container
-        part = msg_iter.next()
-        self.assert_(part.is_multipart())
+        self.assertTrue(parts[0].is_multipart())
 
         # second part should be an inline text/plain attachments with all short
         # fields
-        part = msg_iter.next()
-        self.assert_(not part.is_multipart())
-        self.assertEqual(part.get_content_type(), 'text/plain')
-        self.assertEqual(part.get_content_charset(), 'utf-8')
-        self.assertEqual(part.get_filename(), None)
-        self.assertEqual(part.get_payload(decode=True), '''FirstText: Who
+        self.assertTrue(not parts[1].is_multipart())
+        self.assertEqual(parts[1].get_content_type(), 'text/plain')
+        self.assertEqual(parts[1].get_content_charset(), 'utf-8')
+        self.assertEqual(parts[1].get_filename(), None)
+        self.assertEqual(parts[1].get_payload(decode=True), '''FirstText: Who
 SecondText: What
 ThirdText: I Don't Know
 FourthText: Today
@@ -1431,9 +1429,6 @@ ProblemType: Crash
 Date: now!
 ''')
 
-        # no more parts
-        self.assertRaises(StopIteration, msg_iter.next)
-
     def test_updating(self):
         '''new_keys() and write() with only_new=True.'''
 
